package pdespacho2;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;

public final class Despacho {

	private static File archivo = new File("Despacho.txt");

	/**
	 * Determina la cantidad de registros (lineas) contenidas en
	 * el archivo "Despacho.txt"
	 * @return cantidad de registros
	 */
	public static int numRegistros() {
		int nRegistros = 0;

		try {
			BufferedReader br = new BufferedReader(new FileReader(archivo));
			String linea = br.readLine();
			while (linea != null)
			{
				nRegistros++;
				linea = br.readLine();
			}
			br.close();
		} catch (FileNotFoundException e) {
			System.err.println("No se encontró archivo: " + archivo.getAbsolutePath());
		} catch (IOException e) {
			System.err.println("Error de lectura en archivo: " + archivo.getAbsolutePath());
		}

		return nRegistros;
	}

	/**
	 * Lee los registros del archivo de texto, separa cada dato y los asigna
	 * al vector de nombres y matriz de peso/dimensiones recibidos como parametros.
	 * @param nom Vector tipo String de nombres
	 * @param pesoDimen Matriz tipo double con 1 peso y 3 dimensiones 
	 */
	public static void leerReg(String[] nom, double[][] pesoDimen) {

		int i = 0;//Indice para recorrer vector y matriz

		try {
			BufferedReader br = new BufferedReader(new FileReader(archivo));
			String linea = br.readLine();
			final NumberFormat NUMBER_FORMAT = NumberFormat.getInstance(Locale.FRANCE);
			while (linea != null)
			{
				String[] datos = linea.split(" ");//Separamos datos del registro
				nom[i] = datos[0];//Nombre
				pesoDimen[i][0] = (double) NUMBER_FORMAT.parse(datos[1]);//Peso
				pesoDimen[i][1] = (double) NUMBER_FORMAT.parse(datos[2]);//Dimension 1
				pesoDimen[i][2] = (double) NUMBER_FORMAT.parse(datos[3]);//Dimension 2
				pesoDimen[i][3] = (double) NUMBER_FORMAT.parse(datos[4]);//Dimension 3
				//Pasamos al siguiente registro
				i++;
				linea = br.readLine();
			}
			br.close();
		} catch (FileNotFoundException e) {
			System.err.println("No se encontró archivo: " + archivo.getAbsolutePath());
		} catch (IOException e) {
			System.err.println("Error de lectura en archivo: " + archivo.getAbsolutePath());
		} catch (NumberFormatException e) {
			System.err.println("Error en formato numerico de peso o dimensiones en registro: #" + i);
		} catch (ParseException e) {
			System.err.println("Error en formato numerico de peso o dimensiones en registro: #" + i);

		}	
	}

	public static void mostrarRegistros(String[] nom, double[][] pesoDimen) {

		System.out.println("Nombre     Peso   Alto   Largo  Ancho");
		//Organizamos los nombres:
		for (int i = 0; i < nom.length; i++) {
			System.out.printf("%-8s", nom[i]);
			//Organizamos los registros:
			for (int j = 0; j < pesoDimen[0].length; j++) {
				System.out.printf("%7.2f", pesoDimen[i][j]);
			}
			System.out.println();
		}
	}

	/**
	 * Metodo interno para determinar cual de las tres dimensiones
	 * del paquete es la mayor
	 * @param dimensiones Vector de tipo double con las tres dimensiones
	 * @return Valor double con la dimension mayor
	 */
	private static double getDimensionMayor(double[] dimensiones) {

		double mayor = dimensiones[0];
		if (dimensiones[1] > mayor)
			mayor = dimensiones[1];
		if (dimensiones[2] > mayor)
			mayor = dimensiones[2];
		return mayor;
	}

	/**
	 * Calcula el perimetro del paquete en base a las tres
	 * dimensiones del paquete
	 * @param dim Vector de tipo double con las tres dimensiones
	 * @return Valor tipo double con el perimetro calculado 
	 */
	public static double calculaPerimetro(double[] dim) {

		double mayor = getDimensionMayor(dim);
		return 2d * (dim[0] + dim[1] + dim[2] - mayor);
	}

	/**
	 * Metodo interno que analiza que categoria es aplicable al envio
	 * segun su peso y calcula la tarifa correspondiente
	 * @param peso Peso en kg del paquete
	 * @return Costo del envio segun peso y categoria
	 */
	private static Double calculaTarifa(double peso) {

		if (peso <= 3.50)//Categoria 1
		{
			return 156d * peso;
		}
		else if (peso <= 6.50)//Categoria 2
		{
			return 240 * peso;
		}
		else//Categoria 3
			return 492 * peso;
	}

	/**
	 * Calcula los gastos de envio segun peso y dimensiones.
	 * Además informa de si el envio es EXTRAODINARIO o no lo es.
	 * Ambos datos se devuelven juntos en un vector tipo Object.
	 * Si el peso o las dimensiones exceden los limites de la
	 * tabla de tarifas, el envio se considera rechazado y entonces
	 * devolverá valor 0 para el dato Double del gasto
	 * @param pesoDim vector con peso y tres dimensiones del paquete
	 * @return Vector tipo Object con dos objetos de distinta clase:
	 * Object[0] será un Double con el valor del gasto. Valor 0 si el envio se rechaza.
	 * Object[1] será un Boolean. TRUE indica que es un envio EXTRAORDINARIO
	 */
	public static Object[] calculaGastoEnvio(double[] pesoDim) {

		/*
		 * Necesitamos que este metodo informe de dos cosas:
		 * - El coste de los gastos de envio
		 * - Si el envio es NORMAL o EXTRAORDINARIO
		 * Para ello podemos usar un vector de tipo Object que tendra dos elementos:
		 * El primero sera un objeto Double con el valor del envio calculado
		 * El segundo será un objeto Boolean:
		 * - False nos indicara que es NORMAL, TRUE que es EXTRAORDINARIO
		 */

		Double gasto = 0d;//Aqui calcularemos el gasto. Si el envio es rechazado, quedará en valor 0
		Boolean esExtraordinario = false;//Aqui indicaremos si es un envio extraordinario o no

		//Creamos un vector de apoyo SOLO con las dimensiones para facilitar las siguientes operaciones
		double[] dimensiones = new double[] {pesoDim[1], pesoDim[2], pesoDim[3]};

		//Necesitamos saber la dimension mayor para calcular los gastos.
		double mayor = getDimensionMayor(dimensiones);

		//Calculamos limite para distinguir entre envios NORMALEs o EXTRAORDINARIOS
		double limite = mayor + calculaPerimetro(dimensiones);

		if (pesoDim[0] <= 18d)//Que no supere los 18kg
		{
			//Ninguna dimension supera 1 m
			if (pesoDim[1] <= 1d && pesoDim[2] <= 1d && pesoDim[3] <= 1d && limite <= 2d)
			{//Es un envio normal
				gasto = calculaTarifa(pesoDim[0]);
			}
			else if (pesoDim[1] <= 2d && pesoDim[2] <= 2d && pesoDim[3] <= 2d && limite <= 3d)
			{//Es un envio extraordinario
				esExtraordinario = true;
				gasto = calculaTarifa(pesoDim[0]);
				gasto += (10d * gasto / 100);//Recargo del 10% 
			}
		}
		//Retornamos ambos datos en vector tipo Object
		return new Object[] {gasto, esExtraordinario};
	}

	/**
	 * Muestra en pantalla un listado de los envios con los campos:
	 * Nombre, Tipo de envio y costo
	 * @param nom Vector con los nombres de clientes
	 * @param pesoDimen Matriz con peso y tres dimensiones de cada envio 
	 */
	public static void mostrarListaGastos(String[] nom, double[][] pesoDimen) {

		//Datos a mostrar
		String nombre, envio;
		double costo;

		System.out.println("**Listado de despachos enviados o rechazados**");
		System.out.printf("%-10s %-16s %-8s", "Cliente", " Envio", "Costo");
		System.out.println();
		for (int i = 0; i < nom.length; i++)
		{
			//Recopilamos los tres datos a mostrar
			Object[] datosEnvio = calculaGastoEnvio(pesoDimen[i]);
			nombre = nom[i];
			if ((double)datosEnvio[0] == 0d)//Es un envio rechazado
			{
				envio = "Rechazado";
				costo = 0d;
			}
			else
			{
				costo = (double) datosEnvio[0];//Casting de Object a double
				if ((boolean)datosEnvio[1])//True
					envio = "Extraordinario";
				else
					envio = "Normal";
			}
			//Datos recopilados. Mostramos en pantalla
			System.out.printf("%-10s %-16s %07.2f", nombre, envio, costo);
			System.out.println();
		}
	}

}
